<?php
declare(strict_types=1);

namespace Flat101\Analytics\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Store\Model\ScopeInterface;

class Gtm extends AbstractHelper
{
    CONST PATH_MODULE_ENABLED = 'flat101_analytics/general/enabled';
    CONST PATH_MODULE_GTM_CODE = 'flat101_analytics/general/gtm_code';
    CONST PATH_MODULE_DATALAYER_NAME = 'flat101_analytics/general/datalayer_name';
    CONST DEFAULT_DATALAYER_NAME = 'dataLayer';

    /**
     * Constructor.
     * @param Context $context
     */
    public function __construct(
        Context $context
    ) {
        parent::__construct($context);
    }

    /**
     * Check if GTM is enabled
     *
     * @param null $store_id
     * @return null | string
     */
    public function getGtmEnabled($store_id = null)
    {
        return $this->scopeConfig->getValue(
            self::PATH_MODULE_ENABLED,
            ScopeInterface::SCOPE_STORE,
            $store_id
        );
    }

    /**
     * Get GTM Account code
     *
     * @param null $store_id
     * @return null | string
     */
    public function getGtmAccountCode($store_id = null)
    {
        return $this->scopeConfig->getValue(
            self::PATH_MODULE_GTM_CODE,
            ScopeInterface::SCOPE_STORE,
            $store_id
        );
    }

    /**
     * Check if is enabled from config and has GTM Code
     *
     * @param null $store_id
     * @return bool
     */
    public function isActive($store_id = null)
    {
        $gtmCodeId = $this->getGtmAccountCode($store_id);
        $enabled = $this->getGtmEnabled($store_id);

        return $gtmCodeId && $enabled;
    }

    /**
     * @param null $store_id
     * @return string
     */
    public function getDataLayerName($store_id = null)
    {
        return $this->scopeConfig->getValue(
            self::PATH_MODULE_DATALAYER_NAME,
            ScopeInterface::SCOPE_STORE,
            $store_id) ?? self::DEFAULT_DATALAYER_NAME;
    }

    /**
     * This method is used in XML layout.
     *
     * @param null $store_id
     * @return string
     */
    public function getGtmMainCode($store_id = null): string
    {
        if ($this->isActive($store_id)) {
            return $this->createGtmScript(
                $this->getGtmAccountCode($store_id),
                $this->getDataLayerName($store_id)
            );
        }

        return '';
    }

    /**
     * @param $gtmCode
     * @param $datalayerName
     * @return string
     */
    private function createGtmScript($gtmCode, $datalayerName): string
    {
        return "<!-- Google Tag Manager -->
<script>(function(w,d,s,l,i){w[l]=w[l]||[];w[l].push({'gtm.start':
new Date().getTime(),event:'gtm.js'});var f=d.getElementsByTagName(s)[0],
j=d.createElement(s),dl=l!='dataLayer'?'&l='+l:'';j.async=true;j.src=
'https://www.googletagmanager.com/gtm.js?id='+i+dl;f.parentNode.insertBefore(j,f);
})(window,document,'script','" . $datalayerName . "','" . $gtmCode . "');</script>
<!-- End Google Tag Manager -->";
    }

    /**
     * This method is used in XML layout.
     *
     * @param $store_id
     * @return string
     */
    public function getGtmNoScriptCode($store_id = null): string
    {
        if ($this->isActive($store_id)) {
            return $this->createGtmNoScript($this->getGtmAccountCode($store_id));
        }

        return '';
    }

    /**
     * @param $gtmCode
     * @return string
     */
    private function createGtmNoScript($gtmCode): string
    {
        return "<!-- Google Tag Manager (noscript) -->
<noscript><iframe src='https://www.googletagmanager.com/ns.html?id=" . $gtmCode . "'
height='0' width='0' style='display:none;visibility:hidden'></iframe></noscript>
<!-- End Google Tag Manager (noscript) -->";
    }
}
